#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_LIGHT  2

uint16_t read_adc(uint8_t channel)
{
  ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
  ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
  ADCSRA = 0b11000110;
  //         ||   -+-
  //         ||    |
  //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
  //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
  //         |
  //         +-> Enable ADC (บิต ADEN)
  
  while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
    ;
  return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_d(uint8_t pin)
{
    // read logic level of port-D's pin
    DDRD &= ~(1<<pin);
    return (PIND>>pin) & 1;
} 

void out_d(uint8_t pin, uint8_t val)
{
    // write logic level to port-D's pin
    DDRD |= 1<<pin;
    if (val)
        PORTD |= 1<<pin;
    else
        PORTD &= ~(1<<pin);
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (usbRequest_t *)(void *)data;
    static uint8_t dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_d(led_no+5, !led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_d(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_LIGHT)
    {
      uint16_t l = read_adc(0);
      dataBuffer[0] = l      & 0x00ff;
      dataBuffer[1] = l >> 8 & 0x00ff;
      usbMsgPtr = dataBuffer;
      return 2;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRD = (1<<PD5)|(1<<PD6)|(1<<PD7); // PD5..PD7: Output, PD0-PD4: Input

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
